# axway-flow-sdk

SDK for implementing custom flow nodes for API Builder flows

## Features

* CLI tool for starting a new flow-node plugin
* SDK for building custom flow-node plugins for API Builder flows

## Install

`npm install -g axway-flow-sdk`

## Generating a new flow-node plugin

Generates a new flow-node plugin named "mynode" in the current directory.

```
axway-flow -n mynode
cd api-builder-plugin-fn-mynode
npm install --no-optional
npm run build (optional)
```

## encodeURI example

```js
const sdk = require('axway-flow-sdk');

function getFlowNodes() {
	const flownodes = sdk.init(module)
		.add('encodeURI', {
			name: 'Encode URI',
			icon: 'encode.svg',
			description: 'URI encoder'
		})
		.method('encode', {
			name: 'Encode URI',
			description: 'Encodes a URI by replacing each instance of certain characters with UTF-8 encodings.'
		})
		.parameter('uri', {
			type: 'string',
			description: 'The URI to encode.'
		})
		.output('encoded', {
			name: 'Encoded',
			description: 'The encoded URI.',
			context: '$.encodedURI',
			schema: {
				type: 'string'
			}
		})
		.action((req, cb) => {
			cb.encoded(null, encodeURI(req.params.uri));
		});
	return Promise.resolve(flownodes);
}

exports = module.exports = getFlowNodes;
```

Notice that this encodeURI example is returning a function, `getFlowNodes`, that returns a promise. This allows for the generation of the flow-node to be asynchronous, for example, to allow the loading of content from a remote source. In this example it is used for demostration purposes only, `flownodes` could have been returned from `getFlowNodes` instead.

## Type references

In [Axway API Builder](https://docs.axway.com/bundle/API_Builder_4x_allOS_en/page/api_builder.html),
it is possible to reference other types.  For example, types can be loaded from `./schemas`, registered
from [Models](https://docs.axway.com/bundle/API_Builder_4x_allOS_en/page/models.html), or registered
from service connectors.  Any registered schema can be referenced whenever a `schema` is required.

```js
.parameter('greeting', {
	"$ref": "schema://model/appc.arrowdb/user"
})
.output('next', {
	schema: {
		"$ref": "schema://model/appc.arrowdb/user"
	}
})
```
## Testing your flow-node module

The cli will automatically generate a unit test in `./test/test.js`.  The test will use `mocknode`
to mock an API Builder flow-node.

### mocknode(flownodes) : <code>function</code>
Mocks an API Builder flow-node using the flow-node generated with `NodeBuilder`.

**Access**: public

| Param | Type | Description |
| --- | --- | --- |
| flownodes | <code>NodeBuilder</code> | The `NodeBuilder` object. |

**Testing a successful output callback: `cb.next(null, 'stuff')`**

This example uses mocha and promises to check that the flow-node is defined well enough
to pass the argument `foo` to the method `method`.  It also mocks the callback
using the flow-node's defined `output`, and ensures that the method invokes the correct
callback.

Note if the flow-node is asynchronous, i.e. returns a promise, the promise must be resolved to get the flow-nodes defined in the plugin. This can simply be done using a before block that waits for the resolved promise.

```js
const flownodesPromise = require('../src');
const { mocknode } = require('axway-flow-sdk');

let flownodes;
before(() => {
	return flownodesPromise.then((resolvedFlownodes) => {
		flownodes = resolvedFlownodes;
	});
});

it('describes a test', () => {
	const args = { foo: undefined };
	return mocknode(flownodes).node('example').invoke('method', args)
		.then((response) => {
			expect(response).to.deep.equal({
				next: [null, 'stuff']
			})
		});
})

```

**Testing an error default callback: `cb('invalid argument')`**

This example is similar to the previous example, save that the expectation is that
the method will invoke `cb('invalid argument')` when given an `undefined` parameter.

```js
const flownodes = require('../src');
const { mocknode } = require('axway-flow-sdk');

it('describes a test', () => {
	const args = { foo: undefined };
	return mocknode(flownodes).node('example').invoke('method', args)
		.then((response) => {
			expect(response).to.deep.equal(['invalid argument'])
		});
})
```

# API Reference

<a name="module_axway-flow-sdk..NodeBuilder"></a>

### axway-flow-sdk~NodeBuilder
**Kind**: inner class of [<code>axway-flow-sdk</code>](#module_axway-flow-sdk)  

* [~NodeBuilder](#module_axway-flow-sdk..NodeBuilder)
    * [new NodeBuilder(srcModule, [options])](#new_module_axway-flow-sdk..NodeBuilder_new)
    * [.add(key, [options])](#module_axway-flow-sdk..NodeBuilder+add) ⇒ <code>NodeBuilder</code>
    * [.method(key, [options])](#module_axway-flow-sdk..NodeBuilder+method) ⇒ <code>NodeBuilder</code>
    * [.group(name)](#module_axway-flow-sdk..NodeBuilder+group) ⇒ <code>NodeBuilder</code>
    * [.parameter(name, schema, options)](#module_axway-flow-sdk..NodeBuilder+parameter) ⇒ <code>NodeBuilder</code>
    * [.output(key, [options])](#module_axway-flow-sdk..NodeBuilder+output) ⇒ <code>NodeBuilder</code>
    * [.action(handler)](#module_axway-flow-sdk..NodeBuilder+action) ⇒ <code>NodeBuilder</code>

<a name="new_module_axway-flow-sdk..NodeBuilder_new"></a>

#### new NodeBuilder(srcModule, [options])

| Param | Type | Default | Description |
| --- | --- | --- | --- |
| srcModule | <code>object</code> |  | a node module |
| [options] | <code>object</code> |  | The options for the builder. |
| [options.validate] | <code>object</code> | <code>true</code> | The enables parameter validation.  Disable 	if you need to use refs. |

<a name="module_axway-flow-sdk..NodeBuilder+add"></a>

#### nodeBuilder.add(key, [options]) ⇒ <code>NodeBuilder</code>
Adds a new flownode and prepares the `NodeBuilder` to accept the following flownode
operations:
- [.method(key, [options])](#module_axway-flow-sdk..NodeBuilder+method)
- [.output(key, [options])](#module_axway-flow-sdk..NodeBuilder+output)

The `key` parameter is used to uniquely identify the flownode and represents a distinct
instance of a node for the flow editor.  The `key` will be used as the name unless the
`name` option is provided.  The new node will appear under the "general" category
by default, or under the provided `category` option.

The `icon` option can be bmp, jpeg, png, gif, tiff, or  svg file.  After,
[.method](#module_axway-flow-sdk..NodeBuilder+method)
can be used to add method(s), and
[.output](#module_axway-flow-sdk..NodeBuilder+output) can be used to
define an output.  When done,
[.action](#module_axway-flow-sdk..NodeBuilder+action) can be used to
define an action function and finish the flownode.

**Kind**: instance method of [<code>NodeBuilder</code>](#module_axway-flow-sdk..NodeBuilder)  
**Returns**: <code>NodeBuilder</code> - The current object (this).  
**Access**: public  

| Param | Type | Default | Description |
| --- | --- | --- | --- |
| key | <code>string</code> |  | A unique key identifier for the node. |
| [options] | <code>object</code> |  | Options for the node. |
| [options.name] | <code>string</code> |  | A friendly name for the node as it will appear in the UI. |
| [options.icon] | <code>string</code> |  | An icon file. |
| [options.description] | <code>string</code> |  | A description for the node. |
| [options.category] | <code>string</code> | <code>&quot;general&quot;</code> | A category under which the node will appear 		in the UI (defaults to "general"). |

**Example**  
```js
sdk.init(module).add('encodeURI', { icon: 'encode.svg' });
```
<a name="module_axway-flow-sdk..NodeBuilder+method"></a>

#### nodeBuilder.method(key, [options]) ⇒ <code>NodeBuilder</code>
Adds a new method to the current node flownode and prepares the `NodeBuilder`
to accept the following method operations:
- [.parameter(name, schema, [required])](#module_axway-flow-sdk..NodeBuilder+parameter)
- [.action(handler)](#module_axway-flow-sdk..NodeBuilder+action)

[.add(key, [options])](#module_axway-flow-sdk..NodeBuilder+add) must be
called prior to adding a method.

The `key` uniquely identifies the method for the node and will be used as
the name unless the `name` option is provided.

**Kind**: instance method of [<code>NodeBuilder</code>](#module_axway-flow-sdk..NodeBuilder)  
**Returns**: <code>NodeBuilder</code> - The current object (this).  
**Access**: public  

| Param | Type | Description |
| --- | --- | --- |
| key | <code>string</code> | A unique key identifier for the method. |
| [options] | <code>object</code> | Options for the method. |
| [options.name] | <code>string</code> | A friendly name for the method as it will appear in the UI. |
| [options.description] | <code>string</code> | A description for the method. |

**Example**  
```js
sdk.init(module).add('encodeURI', { icon: 'encode.svg' })
	.method('encode', { name: 'Encode URI' });
```
**Example** *(Disable validation)*  
```js
sdk.init(module, { validation: false }).add('encodeURI', { icon: 'encode.svg' })
	.method('encode', { $ref: '#/definitions/foo' });
```
<a name="module_axway-flow-sdk..NodeBuilder+group"></a>

#### nodeBuilder.group(name) ⇒ <code>NodeBuilder</code>
**Kind**: instance method of [<code>NodeBuilder</code>](#module_axway-flow-sdk..NodeBuilder)  
**Returns**: <code>NodeBuilder</code> - The current object (this).  
**Access**: public  

| Param | Type | Description |
| --- | --- | --- |
| name | <code>string</code> | A unique name for the parameter group as it will appear in the UI. |

**Example**  
```js
sdk.init(module).add('encodeURI', { icon: 'encode.svg' })
	.method('encode', { name: 'Encode URI' })
	.group('Advanced')
	.parameter('uri', { type: 'string' });
```
<a name="module_axway-flow-sdk..NodeBuilder+parameter"></a>

#### nodeBuilder.parameter(name, schema, options) ⇒ <code>NodeBuilder</code>
Adds a new parameter to the current method.  Any number of parameters can be added to a method.

[.method(key, [options])](#module_axway-flow-sdk..NodeBuilder+method) must be
called prior to adding a parameter.

The `name` uniquely identifies the the parameter, and the `schema` is a valid
[JSON Schema](http://json-schema.org) definition (both
[draft-04](http://json-schema.org/draft-04/schema) and
[draft-06](http://json-schema.org/draft-06/schema) are supported).

**Kind**: instance method of [<code>NodeBuilder</code>](#module_axway-flow-sdk..NodeBuilder)  
**Returns**: <code>NodeBuilder</code> - The current object (this).  
**Access**: public  

| Param | Type | Description |
| --- | --- | --- |
| name | <code>string</code> | A unique name for the parameter as it will appear in the UI. |
| schema | <code>object</code> | A schema used to validate the parameter. |
| options | <code>object</code> \| <code>boolean</code> | As an object, specifies additional options to configure the parameter. As a boolean, specifies that the parameter is required. Parameters are required by default. |
| options.required | <code>boolean</code> | specifies that the parameter is required. Parameters are required by default. |
| options.multilineWrapper | <code>object</code> | object that holds the before and after parts that surrounds user provided value in the UI. |
| options.multilineWrapper.before | <code>string</code> | string that is placed before the user provided value in the UI. |
| options.multilineWrapper.after | <code>string</code> | string that is placed after the user provided value in the UI. |
| options.initialType | <code>string</code> | autoselects the input type of the field in the ui |

**Example**  
```js
sdk.init(module).add('encodeURI', { icon: 'encode.svg' })
	.method('encode', { name: 'Encode URI' })
	.parameter('uri', { type: 'string' });
```
<a name="module_axway-flow-sdk..NodeBuilder+output"></a>

#### nodeBuilder.output(key, [options]) ⇒ <code>NodeBuilder</code>
Adds a new output to the current method.  Any number of outputs can be added to a method,
but for usability-sake, you should limit this.  The `output` represents one of the possible
callback routes for your method.  For example, if your method tested prime numbers, then
one output might be `prime`, and the other `not-prime`.

[.method(key, [options])](#module_axway-flow-sdk..NodeBuilder+method)
must be called prior to adding an output.

The `key` uniquely identifies the the output route.  The `schema` is a valid
[JSON Schema](http://json-schema.org) definition (both
[draft-04](http://json-schema.org/draft-04/schema) and
[draft-06](http://json-schema.org/draft-06/schema) are supported).
If `schema` is not provided, then the output type is effectively _any_ type.

The `context` is a valid [JSON Path](https://github.com/json-path/JsonPath) and
is used as the default by the flow editor.  When the output is invoked, the configured
context is where the output value will be written.

**Kind**: instance method of [<code>NodeBuilder</code>](#module_axway-flow-sdk..NodeBuilder)  
**Returns**: <code>NodeBuilder</code> - The current object (this).  
**Access**: public  

| Param | Type | Description |
| --- | --- | --- |
| key | <code>string</code> | A unique key for the output. |
| [options] | <code>object</code> | output options |
| [options.name] | <code>string</code> | A friendly name for the output as it will appear in 		the UI. |
| [options.description] | <code>string</code> | The output description. |
| [options.context] | <code>string</code> | The default context string. |
| [options.schema] | <code>object</code> | The expected JSON schema for the output value. |

**Example**  
```js
sdk.init(module).add('encodeURI', { icon: 'encode.svg' })
	.method('encode', { name: 'Encode URI' })
	.parameter('uri', { type: 'string' })
	.output('encoded', { context: '$.encodedURI', schema: { type: 'string' } });
```
<a name="module_axway-flow-sdk..NodeBuilder+action"></a>

#### nodeBuilder.action(handler) ⇒ <code>NodeBuilder</code>
Assigns an action [`handler`](module:axway-flow-sdk~handler) to
the current method.  The method can only have one action handler.  Assigning an action will
terminate the current method definition.

**Kind**: instance method of [<code>NodeBuilder</code>](#module_axway-flow-sdk..NodeBuilder)  
**Returns**: <code>NodeBuilder</code> - The current object (this).  
**Access**: public  

| Param | Type | Description |
| --- | --- | --- |
| handler | <code>handler</code> | The action [`handler`](module:axway-flow-sdk~handler) 		function. |

**Example**  
```js
sdk.init(module).add('encodeURI', { icon: 'encode.svg' })
	.method('encode', { name: 'Encode URI' })
	.parameter('uri', { type: 'string' })
	.output('encoded', { context: '$.encodedURI', schema: { type: 'string' } })
	.action((req, cb) => cb.encoded(null, encodeURI(req.params.uri));
```
<a name="module_axway-flow-sdk..init"></a>

### axway-flow-sdk~init(module, [options]) ⇒ <code>NodeBuilder</code>
Axway API Builder SDK for creating custom nodes to work with flows.

**Kind**: inner method of [<code>axway-flow-sdk</code>](#module_axway-flow-sdk)  
**Returns**: <code>NodeBuilder</code> - A newly constructed
		[`NodeBuilder`](#module_axway-flow-sdk..NodeBuilder) object  

| Param | Type | Default | Description |
| --- | --- | --- | --- |
| module | <code>object</code> |  | The node module. |
| [options] | <code>object</code> |  | The options for the builder. |
| [options.validate] | <code>object</code> | <code>true</code> | The enables parameter validation.  Disable 	if you need to use refs. |

**Example**  
```js
const sdk = require('axway-flow-sdk');
exports = module.exports = sdk.init(module);
```
<a name="module_axway-flow-sdk..validate"></a>

### axway-flow-sdk~validate(nodes)
Validates the flownodes against the Axway Flow schema.

**Kind**: inner method of [<code>axway-flow-sdk</code>](#module_axway-flow-sdk)  
**Throws**:

- <code>Error</code> If nodes contains invalid flownodes.

**Access**: public  

| Param | Type | Description |
| --- | --- | --- |
| nodes | <code>NodeBuilder</code> | nodes to validate |

**Example**  
```js
const nodes = sdk.init(module).add('encodeURI', { icon: 'encode.svg' })
	.method('encode', { name: 'Encode URI' })
	.parameter('uri', { type: 'string' })
	.output('encoded', { context: '$.encodedURI', schema: { type: 'string' } })
	.action((req, cb) => cb.encoded(null, encodeURI(req.params.uri));
sdk.validate(nodes);
```
<a name="module_axway-flow-sdk..getSelectedOptions"></a>

### axway-flow-sdk~getSelectedOptions(options) ⇒ <code>object</code>
Add user provided option to the parameter schema.

**Kind**: inner method of [<code>axway-flow-sdk</code>](#module_axway-flow-sdk)  
**Returns**: <code>object</code> - selected options and a flag that is set to true if there are selected options.  

| Param | Type | Description |
| --- | --- | --- |
| options | <code>object</code> | Specifies additional options to configure the parameter. |



# Author

Axway <support@axway.com> https://axway.com


# Changes

#### 3.2.0
- #5891: Added alternative way to define parameter options. Instead of `false` as a third argument to `.parameter()`, an object can be provided as `{ required: false }`.
- #5891: Added support for defining wrapper text for multiline and javascript format parameters. This wrapper text will be displayed in the parameter editor but will not be part of the value when saved in the flow.
This is useful for when context or comments are needed.

#### 3.1.11
- #5923: Fixes for both the README.md not being published and the wrong .README.md template.

#### 3.1.10
- #5914: Internal change to re-introduce readme generation from JSDoc.

#### 3.1.9
- #5887: Fix the issue where the `axway-flow -n` command would throw an error

#### 3.1.8
- #5895: Fix issue where new Flow-Node projects would not get generated with a `test` directory.

#### 3.1.7
- #5711: Internal cleanup of npm scripts.

#### 3.1.6
- #5708: Internal changes removing readme autogeneration.

#### 3.1.5
- #5708: Internal changes to mocha configuration.

#### 3.1.4
- #5709: Internal changes to update eslint rules.

#### 3.1.3
- #5707: Internal cleanup to code coverage during build process.

#### 3.1.2
- #5742: Internal change to replace `dot` dependency with `ejs` when generating new flow-node plugins.

#### 3.1.0
- #5600: Support schema references by disabling validation when building flow-nodes.

#### 3.0.0
- #5436: Breaking Change: Minimum supported version of API Builder is now Jakarta
- #5436: Moves advanced HTTP options into a new group, "Advanced HTTP Options"

#### 2.0.9
- #4757: Changed SCM repository and associated internal cleanup.

# License

This code is proprietary, closed source software licensed to you by Axway. All Rights Reserved. You may not modify Axway’s code without express written permission of Axway. You are licensed to use and distribute your services developed with the use of this software and dependencies, including distributing reasonable and appropriate portions of the Axway code and dependencies. Except as set forth above, this code MUST not be copied or otherwise redistributed without express written permission of Axway. This module is licensed as part of the Axway Platform and governed under the terms of the Axway license agreement (General Conditions) located here: [https://support.axway.com/en/auth/general-conditions](https://support.axway.com/en/auth/general-conditions); EXCEPT THAT IF YOU RECEIVED A FREE SUBSCRIPTION, LICENSE, OR SUPPORT SUBSCRIPTION FOR THIS CODE, NOTWITHSTANDING THE LANGUAGE OF THE GENERAL CONDITIONS, AXWAY HEREBY DISCLAIMS ALL SUPPORT AND MAINTENANCE OBLIGATIONS, AS WELL AS ALL EXPRESS AND IMPLIED WARRANTIES, INCLUDING BUT NOT LIMITED TO IMPLIED INFRINGEMENT WARRANTIES, WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE, AND YOU ACCEPT THE PRODUCT AS-IS AND WITH ALL FAULTS, SOLELY AT YOUR OWN RISK. Your right to use this software is strictly limited to the term (if any) of the license or subscription originally granted to you.
